@@ -19,17 +19,20 @@ import os
 import logging
 import tarfile
 import glob
+import operator
 
 from custodian.custodian import ErrorHandler
 from pymatgen.io.vaspio.vasp_input import Poscar, VaspInput
 from pymatgen.transformations.standard_transformations import \
     PerturbStructureTransformation
+from pymatgen.serializers.json_coders import MSONable
+
 from pymatgen.io.vaspio.vasp_output import Vasprun
 from custodian.ansible.intepreter import Modder
 from custodian.ansible.actions import FileActions, DictActions
 
 
-class VaspErrorHandler(ErrorHandler):
+class VaspErrorHandler(ErrorHandler, MSONable):
 
     error_msgs = {
         "tet": ["Tetrahedron method fails for NKPT<4",
@@ -81,7 +84,10 @@ class VaspErrorHandler(ErrorHandler):
             actions.append({'dict': 'KPOINTS',
                             'action': {'_set': {'style': "Gamma"}}})
         if "mesh_symmetry" in self.errors:
-            m = max(vi["KPOINTS"].kpts[0])
+            m = reduce(operator.mul, vi["KPOINTS"].kpts[0])
+            m = max(int(round(m ** (1 / 3))), 1)
+            if vi["KPOINTS"].style.lower().startswith("m"):
+                m += m % 2
             actions.append({'dict': 'KPOINTS',
                             'action': {'_set': {'kpoints': [[m] * 3]}}})
         m = Modder()
@@ -95,8 +101,18 @@ class VaspErrorHandler(ErrorHandler):
     def __str__(self):
         return "Vasp error"
 
+    @property
+    def to_dict(self):
+        return {"@module": self.__class__.__module__,
+                "@class": self.__class__.__name__,
+                "output_filename": self.output_filename}
 
-class UnconvergedErrorHandler(ErrorHandler):
+    @staticmethod
+    def from_dict(d):
+        return VaspErrorHandler(d["output_filename"])
+
+
+class UnconvergedErrorHandler(ErrorHandler, MSONable):
     """
     Check if a run is converged
     """
@@ -131,8 +147,18 @@ class UnconvergedErrorHandler(ErrorHandler):
     def __str__(self):
         return "Run unconverged."
 
+    @property
+    def to_dict(self):
+        return {"@module": self.__class__.__module__,
+                "@class": self.__class__.__name__,
+                "output_filename": self.output_filename}
 
-class PoscarErrorHandler(ErrorHandler):
+    @staticmethod
+    def from_dict(d):
+        return UnconvergedErrorHandler(d["output_filename"])
+
+
+class PoscarErrorHandler(ErrorHandler, MSONable):
 
     def __init__(self, output_filename="vasp.out"):
         self.output_filename = output_filename
@@ -164,6 +190,16 @@ class PoscarErrorHandler(ErrorHandler):
         return {"errors": ["Rotation matrix"],
                 "actions": actions}
 
+    @property
+    def to_dict(self):
+        return {"@module": self.__class__.__module__,
+                "@class": self.__class__.__name__,
+                "output_filename": self.output_filename}
+
+    @staticmethod
+    def from_dict(d):
+        return PoscarErrorHandler(d["output_filename"])
+
 
 def backup():
     error_num = 0
